@@ -19,7 +19,14 @@
 
 #pragma once
 
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Winconsistent-missing-override"
+#endif
 #include <QObject>
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 
 #include <chrono>
 #include <memory>
