@@ -79,7 +79,7 @@ static string tmp_dir = []
 
 // Atomically write contents to filename.
 
-void write_file(string const& filename, string const& contents)
+void write_file(string const& filename, char const* buf, size_t len)
 {
     using namespace boost::filesystem;
 
@@ -97,14 +97,14 @@ void write_file(string const& filename, string const& contents)
     {
         FdPtr fd_ptr(fd, do_close);
 
-        int rc = write(fd_ptr.get(), &contents[0], contents.size());
+        int rc = write(fd_ptr.get(), buf, len);
         if (rc == -1)
         {
             // LCOV_EXCL_START
             throw runtime_error("write_file(): cannot write to \"" + filename + "\": " + safe_strerror(errno));
             // LCOV_EXCL_STOP
         }
-        if (string::size_type(rc) != contents.size())
+        if (size_t(rc) != len)
         {
             throw runtime_error("write_file(): short write for \"" + filename + "\"");  // LCOV_EXCL_LINE
         }
@@ -120,6 +120,16 @@ void write_file(string const& filename, string const& contents)
     }
 }
 
+void write_file(string const& filename, string const& contents)
+{
+    write_file(filename, contents.data(), contents.size());
+}
+
+void write_file(string const& filename, QByteArray const& contents)
+{
+    write_file(filename, contents.constData(), contents.size());
+}
+
 // Write contents of in_fd to out_fd, using current read position of in_fd.
 
 void write_file(int in_fd, int out_fd)
@@ -149,7 +159,7 @@ void write_file(int in_fd, int out_fd)
 
 // Write contents of fd to path.
 
-void write_file(int fd, string const& path)
+void write_file(string const& path, int fd)
 {
     FdPtr out_fd(open(path.c_str(), O_WRONLY | O_TRUNC | O_CREAT | O_CLOEXEC, 0600), do_close);
     if (out_fd.get() == -1)
