@@ -16,8 +16,9 @@
  * Authored by: James Henstridge <james.henstridge@canonical.com>
  */
 
-#include <settings.h>
+#include <internal/settings.h>
 
+#include <internal/env_vars.h>
 #include "settings-defaults.h"
 
 #pragma GCC diagnostic push
@@ -27,6 +28,7 @@
 #pragma GCC diagnostic pop
 #include <QDebug>
 
+#include <chrono>
 #include <memory>
 
 using namespace std;
@@ -37,6 +39,9 @@ namespace unity
 namespace thumbnailer
 {
 
+namespace internal
+{
+
 Settings::Settings()
     : Settings("com.canonical.Unity.Thumbnailer")
 {
@@ -90,9 +95,18 @@ int Settings::retry_not_found_hours() const
     return get_positive_int("retry-not-found-hours", RETRY_NOT_FOUND_HOURS_DEFAULT);
 }
 
-int Settings::retry_error_hours() const
+int Settings::retry_error_max_seconds() const
 {
-    return get_positive_int("retry-error-hours", RETRY_ERROR_HOURS_DEFAULT);
+    auto retry_error_hours = get_positive_int("retry-error-hours", RETRY_ERROR_HOURS_DEFAULT);
+    chrono::seconds retry_max = chrono::duration_cast<chrono::seconds>(chrono::hours(retry_error_hours));
+    chrono::seconds retry_min = chrono::seconds(2 * extraction_timeout());
+    if (retry_max < retry_min)
+    {
+        retry_max = retry_min;
+        qWarning() << "retry-error-hours setting is < 2 * extraction-timeout. Adjusted max retry backoff to"
+                   << retry_max.count() << "seconds.";
+    }
+    return retry_max.count();
 }
 
 int Settings::max_downloads() const
@@ -120,6 +134,36 @@ bool Settings::trace_client() const
     return get_bool("trace-client", TRACE_CLIENT_DEFAULT);
 }
 
+int Settings::log_level() const
+{
+    using namespace unity::thumbnailer::internal;
+
+    int log_level = get_positive_or_zero_int("log-level", LOG_LEVEL_DEFAULT);
+    char const* level = getenv(LOG_LEVEL);
+    if (level && *level)
+    {
+        int l;
+        try
+        {
+            l = std::stoi(level);
+        }
+        catch (std::exception const& e)
+        {
+            qCritical() << "Environment variable" << LOG_LEVEL << "has invalid setting:" << level
+                        << "(expected value in range 0..2) - variable ignored";
+            return log_level;
+        }
+        if (l < 0 || l > 2)
+        {
+            qCritical() << "Environment variable" << LOG_LEVEL << "has invalid setting:" << level
+                        << "(expected value in range 0..2) - variable ignored";
+            return log_level;
+        }
+        log_level = l;
+    }
+    return log_level;
+}
+
 string Settings::get_string(char const* key, string const& default_value) const
 {
     if (!settings_ || !g_settings_schema_has_key(schema_.get(), key))
@@ -179,6 +223,8 @@ bool Settings::get_bool(char const* key, bool default_value) const
     return g_settings_get_boolean(settings_.get(), key);
 }
 
+}  // namespace internal
+
 }  // namespace thumbnailer
 
 }  // namespace unity
