@@ -23,6 +23,8 @@
 #include <internal/safe_strerror.h>
 #include "util.h"
 
+#include <boost/filesystem.hpp>
+
 #include <cassert>
 
 #include <fcntl.h>
@@ -68,8 +70,31 @@ GetLocalThumbnail::GetLocalThumbnail(QCommandLineParser& parser)
     {
         throw parser.helpText();
     }
+
     input_path_ = args[1];
+    if (input_path_.isEmpty())
+    {
+        throw QString("GetLocalThumbnail(): invalid empty input path");
+    }
+
+    // Remote end needs an absolute path.
+    if (!boost::filesystem::path(input_path_.toStdString()).is_absolute())
+    {
+        try
+        {
+            input_path_ = QString::fromStdString(boost::filesystem::canonical(input_path_.toStdString()).native());
+        }
+        catch (std::exception const&)
+        {
+            // If name can't be canonicalised, errors will be dealt with on the server side.
+        }
+    }
+
     output_dir_ = args.size() == 3 ? args[2] : current_directory();
+    if (output_dir_.isEmpty())
+    {
+        throw QString("GetLocalThumbnail(): invalid empty output directory");
+    }
 
     if (parser.isSet(size_option))
     {
@@ -95,10 +120,10 @@ void GetLocalThumbnail::run(DBusConnection& conn)
         {
             throw reply.error().message();  // LCOV_EXCL_LINE
         }
-        QDBusUnixFileDescriptor thumbnail_fd = reply.value();
+        QByteArray thumbnail = reply.value();
 
         string out_path = make_output_path(input_path_.toStdString(), size_, output_dir_.toStdString());
-        write_file(thumbnail_fd.fileDescriptor(), out_path);
+        write_file(out_path, thumbnail);
     }
     catch (std::exception const& e)
     {
