@@ -21,6 +21,8 @@
 #include <QSize>
 #include <QString>
 
+class QByteArray;
+
 namespace unity
 {
 
@@ -42,6 +44,9 @@ QString current_directory();
 
 std::string make_output_path(std::string const& inpath, QSize const& size, std::string const& dir);
 
+// Convert the image in ba to PNG (if it isn't PNG already).
+void to_png(QByteArray& ba);
+
 }  // namespace tools
 
 }  // namespace thumbnailer
