@@ -25,20 +25,16 @@
 #include <internal/check_access.h>
 #include <internal/image.h>
 #include <internal/imageextractor.h>
+#include <internal/local_album_art.h>
 #include <internal/make_directories.h>
+#include <internal/mimetype.h>
 #include <internal/raii.h>
 #include <internal/safe_strerror.h>
+#include <internal/settings.h>
 #include <internal/ubuntuserverdownloader.h>
-#include <settings.h>
+#include <internal/version.h>
 
 #include <boost/filesystem.hpp>
-
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#pragma GCC diagnostic ignored "-Wold-style-cast"
-#include <gio/gio.h>
-#pragma GCC diagnostic pop
-
 #include <unity/UnityExceptions.h>
 
 #include <fcntl.h>
@@ -78,20 +74,24 @@ class RequestBase : public ThumbnailRequest
 #endif
 public:
     virtual ~RequestBase() = default;
-    string thumbnail() override;
+    QByteArray thumbnail() override;
     FetchStatus status() const override;
+
     string const& key() const override
     {
         return key_;
     }
+
     void check_client_credentials(uid_t, std::string const&) override
     {
     }
+
     enum class CachePolicy
     {
         cache_fullsize,
         dont_cache_fullsize
     };
+
     struct ImageData
     {
         FetchStatus status;
@@ -99,13 +99,6 @@ public:
         CachePolicy cache_policy;
         Location location;
 
-        ImageData()
-            : status(FetchStatus::error)
-            , cache_policy(CachePolicy::dont_cache_fullsize)
-            , location(Location::local)
-        {
-        }
-
         ImageData(Image const& image, CachePolicy policy, Location location)
             : status(FetchStatus::downloaded)
             , image(image)
@@ -114,9 +107,10 @@ public:
         {
         }
 
-        ImageData(FetchStatus status, Location location)
+        ImageData(FetchStatus status, CachePolicy policy, Location location)
             : status(status)
-            , cache_policy(CachePolicy::dont_cache_fullsize)
+            , image(Image())
+            , cache_policy(policy)
             , location(location)
         {
         }
@@ -125,18 +119,29 @@ public:
         ImageData& operator=(ImageData&&) = default;
     };
 
+    void set_error_message(string const& msg)
+    {
+        error_message_ = msg;
+    }
+
+    string error_message() const
+    {
+        return error_message_;
+    }
+
 protected:
     RequestBase(Thumbnailer* thumbnailer,
                 string const& key,
                 QSize const& requested_size,
                 chrono::milliseconds timeout);
-    virtual ImageData fetch(QSize const& size_hint) = 0;
+    virtual ImageData fetch(QSize const& size_hint) noexcept = 0;
 
     ArtDownloader* downloader() const
     {
         return thumbnailer_->downloader_.get();
     }
 
+    // LCOV_EXCL_START
     string printable_key() const
     {
         // Substitute "\\0" for all occurrences of '\0' in key_.
@@ -155,13 +160,12 @@ protected:
         }
         return new_key;
     }
+    // LCOV_EXCL_STOP
 
     Thumbnailer* thumbnailer_;
     string key_;
-    // TODO: Make this a const member again once we remove the hack
-    //       that adjusts invalid QSizes to 128x128.
-    //QSize const requested_size_;
-    QSize requested_size_;
+    QSize const requested_size_;
+    string error_message_;
     chrono::milliseconds timeout_;
 
 private:
@@ -189,7 +193,7 @@ public:
     void check_client_credentials(uid_t user, std::string const& label) override;
 
 protected:
-    ImageData fetch(QSize const& size_hint) override;
+    ImageData fetch(QSize const& size_hint) noexcept override;
     void download(std::chrono::milliseconds timeout) override;
 
 private:
@@ -215,7 +219,7 @@ public:
                  chrono::milliseconds timeout);
 
 protected:
-    ImageData fetch(QSize const& size_hint) override;
+    ImageData fetch(QSize const& size_hint) noexcept override;
     void download(std::chrono::milliseconds timeout) override;
 
 private:
@@ -242,7 +246,7 @@ public:
                   chrono::milliseconds timeout);
 
 protected:
-    ImageData fetch(QSize const& size_hint) override;
+    ImageData fetch(QSize const& size_hint) noexcept override;
     void download(std::chrono::milliseconds timeout) override;
 
 private:
@@ -288,20 +292,21 @@ RequestBase::RequestBase(Thumbnailer* thumbnailer,
 // size, store the scaled version to the thumbnail cache and return
 // it.
 
-string RequestBase::thumbnail()
+QByteArray RequestBase::thumbnail()
 {
     try
     {
-        // TODO: Turn this into an error soonish. This is logged here as well
-        //       as on the Qt/QML size so we don't need to look at the logs for
-        //       all applications to spot when we get an invalid size.
+        // This is logged here as well as on the Qt/QML side because,
+        // at least in theory, someone could talk to the DBus interface
+        // directly without going through the official client-side API.
         if (!requested_size_.isValid())
         {
-            qWarning().nospace() << "deprecated invalid size: "
-                                 << requested_size_
-                                 << ". This feature will be removed soon. Pass the desired size instead.";
-            requested_size_.setWidth(128);
-            requested_size_.setHeight(128);
+            QString msg;
+            QTextStream s(&msg, QIODevice::WriteOnly);
+            s << "invalid size: " << "(" << requested_size_.width() << "," << requested_size_.height()
+              << ")";
+            qWarning().nospace().noquote() << msg << ", key = " << QString::fromStdString(printable_key());
+            throw invalid_argument(msg.toStdString());
         }
 
         // Enforce size limitation.
@@ -322,11 +327,13 @@ string RequestBase::thumbnail()
         sized_key += to_string(target_size.height());
 
         // Check if we have the thumbnail in the cache already.
+        assert(thumbnailer_);
+        assert(thumbnailer_->thumbnail_cache_);
         auto thumbnail = thumbnailer_->thumbnail_cache_->get(sized_key);
         if (thumbnail)
         {
             status_ = FetchStatus::cache_hit;
-            return *thumbnail;
+            return QByteArray::fromStdString(*thumbnail);
         }
 
         // Don't have the thumbnail yet, see if we have the original image around.
@@ -336,6 +343,7 @@ string RequestBase::thumbnail()
         {
             status_ = ThumbnailRequest::FetchStatus::scaled_from_fullsize;
             scaled_image = Image(*full_size, target_size);
+            full_size = "";  // Release memory
         }
         else
         {
@@ -348,55 +356,30 @@ string RequestBase::thumbnail()
                 status_ = ThumbnailRequest::FetchStatus::cached_failure;
                 return "";
             }
-            ImageData image_data;
-            try
-            {
-                image_data = fetch(target_size);
-                status_ = image_data.status;
-            }
-            catch (...)
-            {
-                // Record any failure due to an exception. Because GStreamer
-                // is flaky and can hang, if we extracted via vs-thumb and got
-                // an error, we don't add the failure to the failure cache
-                // because the next attempt may work again. For extraction
-                // via vs-thumb, location is local and policy is cache_fullsize.
-                if (image_data.location == Location::local)
-                {
-                    if (image_data.cache_policy != CachePolicy::cache_fullsize)
-                    {
-                        thumbnailer_->failure_cache_->put(key_, "");  // It wasn't vs-thumb that failed.
-                    }
-                }
-                else
-                {
-                    // TODO: This appears to be impossible to cover because because fetch()
-                    //       does not throw when a remote access fails. Consider unifying
-                    //       the error handling in ImageData to do never throw at all,
-                    //       so everthing can be handled via the ImageData::status member;
-                    thumbnailer_->nw_fail_time_ = chrono::system_clock::now();
-                }
-                image_data.status = FetchStatus::error;
-                throw;
-            }
+
+            ImageData image_data = fetch(target_size);
+            status_ = image_data.status;
             switch (status_)
             {
                 case FetchStatus::downloaded:      // Success, we'll return the thumbnail below.
+                    if (image_data.location == Location::remote)
+                    {
+                        thumbnailer_->backoff_.reset();
+                    }
                     break;
                 case FetchStatus::needs_download:  // Caller will call download().
                     if (image_data.location == Location::remote)
                     {
-                        // If we had the last failure within the retry limit,
-                        // don't attempt the download.
-                        auto now = chrono::system_clock::now();
-                        if (now <= thumbnailer_->nw_fail_time_ + chrono::hours(thumbnailer_->retry_error_hours_))
+                        // If we had the last failure within the retry limit, don't attempt the download.
+                        if (!thumbnailer_->backoff_.retry_ok())
                         {
-                            status_ = ThumbnailRequest::FetchStatus::cached_failure;
+                            qWarning() << "RequestBase::thumbnail(): server access retry time not reached yet";
+                            status_ = ThumbnailRequest::FetchStatus::temporary_error;
                         }
                     }
                     return "";
-                case FetchStatus::no_network:      // Network down, try again next time.
-                    return "";
+                case FetchStatus::network_down:    // Try again next time.
+                    return "";  // LCOV_EXCL_LINE  // No way to test this without physically disabling network.
                 case FetchStatus::not_found:
                 {
                     // Authoritative answer that artwork does not exist.
@@ -409,30 +392,48 @@ string RequestBase::thumbnail()
                         later = chrono::system_clock::now() + chrono::hours(thumbnailer_->retry_not_found_hours_);
                     }
                     thumbnailer_->failure_cache_->put(key_, "", later);
+                    if (image_data.location == Location::remote)
+                    {
+                        // Even though we didn't get an image, the request itself worked.
+                        thumbnailer_->backoff_.reset();
+                    }
                     return "";
                 }
-                case FetchStatus::error:
+                case FetchStatus::hard_error:
                 {
-                    // Some non-authoritative failure, such as the server not responding,
-                    // or an out-of-process codec reporting an error.
-                    // For local files, we don't set an expiry time because, if the file is
-                    // changed, (say, its permissions change), the file's key will change too.
-                    // For remote files, we record the time of this failure.
-                    if (image_data.location == Location::local)
+                    // No chance of recovery, the problem is with the request data.
+                    thumbnailer_->failure_cache_->put(key_, "");
+                    if (image_data.location == Location::remote)
                     {
-                        // TODO: This is currently uncovered because we handle this earlier,
-                        //       when fetch() throws. Consider unifying
-                        //       the error handling in ImageData to do never throw at all,
-                        //       so everthing can be handled here.
-                        thumbnailer_->failure_cache_->put(key_, "");
+                        // Even though we didn't get an image, the request itself worked.
+                        thumbnailer_->backoff_.reset();
                     }
-                    else
+                    return "";
+                }
+                case FetchStatus::temporary_error:
+                {
+                    // Extraction failure for local media is always a hard error.
+                    assert(image_data.location == Location::remote);
+
+                    // Some non-authoritative failure, such as the server not responding.
+                    if (thumbnailer_->backoff_.adjust_retry_limit())
                     {
-                        thumbnailer_->nw_fail_time_ = chrono::system_clock::now();
+                        qWarning() << "RequestBase::thumbnail(): unexpected download error, disabling server access for"
+                                   << thumbnailer_->backoff_.backoff_period().count() << "seconds";
                     }
-                    // TODO: That's really poor. Should store the error data so we can produce
-                    //       a proper diagnostic.
-                    throw runtime_error("fetch() failed");
+                    return "";
+                }
+                case FetchStatus::timeout:
+                {
+                    // Extraction for local media never returns a timeout error.
+                    assert(image_data.location == Location::remote);
+
+                    // We do not treat remote request timeout as a temporary error because the
+                    // remote server can be slow when 7digital is slow, which happens quite often.
+                    qWarning() << "RequestBase::thumbnail(): request timed out";
+                    // No backoff reset here. Timeouts are like requests that don't return an image,
+                    // except that timeout is not authoritative.
+                    return "";
                 }
                 default:
                     abort();  // LCOV_EXCL_LINE  // Impossible
@@ -445,31 +446,41 @@ string RequestBase::thumbnail()
             // artwork a second time.
             if (image_data.cache_policy == CachePolicy::cache_fullsize)
             {
-                Image full_size_image = image_data.image;
-                auto w = full_size_image.width();
-                auto h = full_size_image.height();
+                auto w = image_data.image.width();
+                auto h = image_data.image.height();
                 auto max_size = thumbnailer_->max_size_;
                 if (max(w, h) > max_size)
                 {
                     // Don't put ridiculously large images into the full-size cache.
-                    full_size_image = full_size_image.scale(QSize(max_size, max_size));
+                    // We scale in place here to release memory immediately. This
+                    // might cause a second scaling operation below, but the
+                    // quality loss is negligible when scaling from something larger
+                    // than max_size to max_size, and then scaling a second time.
+                    image_data.image = image_data.image.scale(QSize(max_size, max_size));
                 }
-                thumbnailer_->full_size_cache_->put(key_, full_size_image.to_jpeg(90));  // Keep high-quality image.
+                // Keep high-quality image.
+                thumbnailer_->full_size_cache_->put(key_, image_data.image.jpeg_or_png_data(90));
             }
             // If the image is already within the target dimensions, this
             // will be a no-op.
             scaled_image = image_data.image.scale(target_size);
+            image_data.image = Image();
         }
 
-        string jpeg = scaled_image.to_jpeg();
-        thumbnailer_->thumbnail_cache_->put(sized_key, jpeg);
-        return jpeg;
+        string data = scaled_image.jpeg_or_png_data();
+        scaled_image = Image();
+        thumbnailer_->thumbnail_cache_->put(sized_key, data);
+        return QByteArray::fromStdString(data);
     }
+    // LCOV_EXCL_START
     catch (std::exception const& e)
     {
-        status_ = FetchStatus::error;
-        throw unity::ResourceException("RequestBase::thumbnail(): key = " + printable_key());
+        // Something totally unexpected happened.
+        string msg = "RequestBase::thumbnail(): key = " + printable_key() + ": " + e.what();
+        qCritical() << QString::fromStdString(msg);
+        throw unity::ResourceException(msg);
     }
+    // LCOV_EXCL_STOP
 }
 
 ThumbnailRequest::FetchStatus RequestBase::status() const
@@ -484,6 +495,11 @@ LocalThumbnailRequest::LocalThumbnailRequest(Thumbnailer* thumbnailer,
     : RequestBase(thumbnailer, "", requested_size, timeout)
     , filename_(filename)
 {
+    if (!boost::filesystem::path(filename).is_absolute())
+    {
+        throw runtime_error("LocalThumbnailRequest(): " + filename_ + ": file name must be an absolute path");
+    }
+
     // We canonicalise the path name both to avoid caching the file
     // multiple times, and to ensure our access checks are against the
     // real file rather than a symlink.
@@ -536,56 +552,58 @@ void LocalThumbnailRequest::check_client_credentials(uid_t user,
 
 }
 
-
-RequestBase::ImageData LocalThumbnailRequest::fetch(QSize const& size_hint)
+RequestBase::ImageData LocalThumbnailRequest::fetch(QSize const& size_hint) noexcept
 {
-    if (image_extractor_)
+    // Default in case something below throws.
+    ImageData image_data(FetchStatus::hard_error, CachePolicy::dont_cache_fullsize, Location::local);
+    try
     {
-        // The image data has been extracted via vs-thumb
-        auto id = ImageData(Image(image_extractor_->data()), CachePolicy::cache_fullsize, Location::local);
-        return id;
-    }
-
-    // Work out content type.
-    gobj_ptr<GFile> file(g_file_new_for_path(filename_.c_str()));
-    assert(file);  // Cannot fail according to doc.
-
-    gobj_ptr<GFileInfo> info(g_file_query_info(file.get(), G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
-                                               G_FILE_QUERY_INFO_NONE,
-                                               /* cancellable */ NULL,
-                                               /* error */ NULL));  // TODO: need decent error reporting
-    if (!info)
-    {
-        return ImageData(FetchStatus::error, Location::local);  // LCOV_EXCL_LINE
-    }
-
-    string content_type = g_file_info_get_attribute_string(info.get(), G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE);
-    if (content_type.empty())
-    {
-        return ImageData(FetchStatus::error, Location::local);  // LCOV_EXCL_LINE
-    }
-
-    // Call the appropriate image extractor and return the image data as JPEG (not scaled).
-    // We indicate that full-size images are to be cached only for audio and video files,
-    // for which extraction is expensive. For local images, we don't cache full size.
-
-    if (content_type.find("audio/") == 0 || content_type.find("video/") == 0)
-    {
-        return ImageData(FetchStatus::needs_download, Location::local);
-    }
-    if (content_type.find("image/") == 0)
-    {
-        FdPtr fd(open(filename_.c_str(), O_RDONLY | O_CLOEXEC), do_close);
-        if (fd.get() < 0)
+        if (image_extractor_)
         {
-            // LCOV_EXCL_START
-            throw runtime_error("LocalThumbnailRequest::fetch(): Could not open " + filename_ + ": " + safe_strerror(errno));
-            // LCOV_EXCL_STOP
+            // The image data has been extracted via vs-thumb. Update image_data policy in case read() throws.
+            image_data.cache_policy = CachePolicy::cache_fullsize;
+            return ImageData(Image(image_extractor_->read()), CachePolicy::cache_fullsize, Location::local);
+        }
+
+        string content_type = get_mimetype(filename_);
+        assert(!content_type.empty());
+
+        // Call the appropriate image extractor and return the image data as JPEG (not scaled).
+        // We indicate that full-size images are to be cached only for video files,
+        // for which extraction is expensive. For local audio and images, we don't cache full size.
+
+        if (content_type.find("image/") == 0)
+        {
+            FdPtr fd(open(filename_.c_str(), O_RDONLY | O_CLOEXEC), do_close);
+            if (fd.get() < 0)
+            {
+                // LCOV_EXCL_START
+                throw runtime_error("LocalThumbnailRequest::fetch(): Could not open " + filename_ + ": " + safe_strerror(errno));
+                // LCOV_EXCL_STOP
+            }
+            Image scaled(fd.get(), size_hint);
+            return ImageData(scaled, CachePolicy::dont_cache_fullsize, Location::local);
+        }
+        else if (content_type.find("audio/") == 0)
+        {
+            string art = extract_local_album_art(filename_);
+            if (!art.empty())
+            {
+                return ImageData(Image(art), CachePolicy::dont_cache_fullsize, Location::local);
+            }
+        }
+        else if (content_type.find("video/") == 0)
+        {
+            return ImageData(FetchStatus::needs_download, CachePolicy::cache_fullsize, Location::local);
         }
-        Image scaled(fd.get(), size_hint);
-        return ImageData(scaled, CachePolicy::dont_cache_fullsize, Location::local);
     }
-    return ImageData(FetchStatus::not_found, Location::local);
+    catch (std::exception const& e)
+    {
+        qCritical().nospace() << "LocalThumbnailRequest::fetch(): " << e.what();
+        set_error_message(e.what());
+        return image_data;
+    }
+    return ImageData(FetchStatus::not_found, image_data.cache_policy, Location::local);
 }
 
 void LocalThumbnailRequest::download(chrono::milliseconds timeout)
@@ -617,35 +635,62 @@ namespace
 // Logic for AlbumRequest::fetch() and ArtistRequest::fetch() is the same,
 // so we use this helper function for both.
 
-RequestBase::ImageData common_fetch(shared_ptr<ArtReply> const& artreply)
+RequestBase::ImageData common_fetch(RequestBase* request, shared_ptr<ArtReply> const& artreply) noexcept
 {
+    assert(request);
+
+    RequestBase::ImageData image_data(RequestBase::FetchStatus::needs_download,
+                                      RequestBase::CachePolicy::cache_fullsize,
+                                      Location::remote);
     if (!artreply)
     {
-        return RequestBase::ImageData(RequestBase::FetchStatus::needs_download, Location::remote);
+        return image_data;
     }
-    if (artreply->succeeded())
+    request->set_error_message(artreply->error_string().toStdString());  // Default for below.
+    switch (artreply->status())
     {
-        auto raw_data = artreply->data();
-        Image full_size(string(raw_data.data(), raw_data.size()));
-        return RequestBase::ImageData(full_size, RequestBase::CachePolicy::cache_fullsize, Location::remote);
+        case ArtReply::Status::success:
+        {
+            try
+            {
+                auto raw_data = artreply->data();
+                Image full_size(string(raw_data.constData(), raw_data.size()));
+                return RequestBase::ImageData(full_size, RequestBase::CachePolicy::cache_fullsize, Location::remote);
+            }
+            catch (std::exception const& e)
+            {
+                request->set_error_message(e.what());
+                image_data.status = RequestBase::FetchStatus::hard_error;
+                return image_data;
+            }
+        }
+        case ArtReply::Status::not_found:
+            image_data.status = RequestBase::FetchStatus::not_found;
+            return image_data;
+        case ArtReply::Status::temporary_error:
+            image_data.status = RequestBase::FetchStatus::temporary_error;
+            return image_data;
+        case ArtReply::Status::hard_error:
+            image_data.status = RequestBase::FetchStatus::hard_error;
+            return image_data;
+        case ArtReply::Status::timeout:
+            image_data.status = RequestBase::FetchStatus::timeout;
+            return image_data;
+        case ArtReply::Status::network_down:
+            // LCOV_EXCL_START
+            image_data.status = RequestBase::FetchStatus::network_down;
+            return image_data;
+            // LCOV_EXCL_STOP
+        default:
+            abort();  // LCOV_EXCL_LINE  // Impossible
     }
-    qDebug() << "common_fetch(): Request failed:" << artreply->error_string();
-    if (artreply->not_found_error())
-    {
-        return RequestBase::ImageData(RequestBase::FetchStatus::not_found, Location::remote);
-    }
-    if (artreply->network_down())
-    {
-        return RequestBase::ImageData(RequestBase::FetchStatus::no_network, Location::remote);
-    }
-    return RequestBase::ImageData(RequestBase::FetchStatus::error, Location::remote);
 }
 
 }  // namespace
 
-RequestBase::ImageData AlbumRequest::fetch(QSize const& /*size_hint*/)
+RequestBase::ImageData AlbumRequest::fetch(QSize const& /*size_hint*/) noexcept
 {
-    return common_fetch(artreply_);
+    return common_fetch(this, artreply_);
 }
 
 void AlbumRequest::download(chrono::milliseconds timeout)
@@ -669,9 +714,9 @@ ArtistRequest::ArtistRequest(Thumbnailer* thumbnailer,
 {
 }
 
-RequestBase::ImageData ArtistRequest::fetch(QSize const& /*size_hint*/)
+RequestBase::ImageData ArtistRequest::fetch(QSize const& /*size_hint*/) noexcept
 {
-    return common_fetch(artreply_);
+    return common_fetch(this, artreply_);
 }
 
 void ArtistRequest::download(chrono::milliseconds timeout)
@@ -681,31 +726,25 @@ void ArtistRequest::download(chrono::milliseconds timeout)
         timeout = timeout_;
     }
     artreply_ = downloader()->download_artist(QString::fromStdString(artist_), QString::fromStdString(album_), timeout);
-    connect(artreply_.get(), &ArtReply::finished, this, &ThumbnailRequest::downloadFinished, Qt::DirectConnection);
+    connect(artreply_.get(), &ArtReply::finished, this, &ArtistRequest::downloadFinished, Qt::DirectConnection);
 }
 
 namespace
 {
 
-// Key under which we store the last time we had a network failure.
-// No entry in the failure cache ever has this key because keys for
+// Keys under which we store the last time we had a network failure
+// and the current backoff period (in seconds).
+// No entry in the failure cache ever has these keys because keys for
 // local files include inode number and modification times.
 string const LAST_NETWORK_FAIL_TIME_KEY = "/*** LAST_NETWORK_FAIL_TIME ***/";
+string const BACKOFF_PERIOD_KEY = "/*** BACKOFF_PERIOD ***/";
 
 }
 
 Thumbnailer::Thumbnailer()
     : downloader_(new UbuntuServerDownloader())
 {
-    string xdg_base = g_get_user_cache_dir();
-    if (xdg_base == "")
-    {
-        // LCOV_EXCL_START
-        string s("Thumbnailer(): Could not determine cache dir.");
-        throw runtime_error(s);
-        // LCOV_EXCL_STOP
-    }
-
+    string xdg_base = g_get_user_cache_dir();  // Always returns something, even HOME and XDG_CACHE_HOME are not set.
     string cache_dir = xdg_base + "/unity-thumbnailer";
     make_directories(cache_dir, 0700);
 
@@ -723,18 +762,28 @@ Thumbnailer::Thumbnailer()
                                                           core::CacheDiscardPolicy::lru_ttl));
         max_size_ = settings.max_thumbnail_size();
         retry_not_found_hours_ = settings.retry_not_found_hours();
-        retry_error_hours_ = settings.retry_error_hours();
         extraction_timeout_ = chrono::milliseconds(settings.extraction_timeout() * 1000);
+        backoff_.set_min_backoff(chrono::seconds(settings.extraction_timeout() * 2));
+        backoff_.set_max_backoff(chrono::seconds(settings.retry_error_max_seconds()));
 
         // For transient remote errors, we read the time at which the last failure
-        // happened. The destructor writes that time back out, so we remember
-        // the time across re-starts. We call contains_key() to avoid generating
+        // happened and the backoff period. The destructor writes these values back out,
+        // so we remember the values across re-starts. We call contains_key() to avoid generating
         // a bogus cache miss in the stats.
         if (failure_cache_->contains_key(LAST_NETWORK_FAIL_TIME_KEY))
         {
-            auto last_network_fail_time = failure_cache_->get(LAST_NETWORK_FAIL_TIME_KEY);
-            nw_fail_time_ = chrono::system_clock::time_point(chrono::seconds(stoll(*last_network_fail_time)));
+            auto last_fail_time = failure_cache_->get(LAST_NETWORK_FAIL_TIME_KEY);
+            auto fail_time_point = chrono::system_clock::time_point(chrono::seconds(stoll(*last_fail_time)));
+            backoff_.set_last_fail_time(fail_time_point);
         }
+        if (failure_cache_->contains_key(BACKOFF_PERIOD_KEY))
+        {
+            auto backoff = failure_cache_->get(BACKOFF_PERIOD_KEY);
+            backoff_.set_backoff_period(chrono::seconds(stoll(*backoff)));
+        }
+
+        // Apply any adjustments to the caches that are version-dependent.
+        apply_upgrade_actions(cache_dir);
     }
     catch (std::exception const& e)
     {
@@ -746,8 +795,10 @@ Thumbnailer::~Thumbnailer()
 {
     try
     {
-        auto seconds = chrono::duration_cast<chrono::seconds>(nw_fail_time_.time_since_epoch()).count();
+        auto seconds = chrono::duration_cast<chrono::seconds>(backoff_.last_fail_time().time_since_epoch()).count();
         failure_cache_->put(LAST_NETWORK_FAIL_TIME_KEY, to_string(seconds));
+        seconds = backoff_.backoff_period().count();
+        failure_cache_->put(BACKOFF_PERIOD_KEY, to_string(seconds));
     }
     // LCOV_EXCL_START
     catch (std::exception const& e)
@@ -761,10 +812,28 @@ Thumbnailer::~Thumbnailer()
     // LCOV_EXCL_STOP
 }
 
+void Thumbnailer::apply_upgrade_actions(string const& cache_dir)
+{
+    Version v(cache_dir);
+    if (v.prev_cache_version() != v.cache_version)
+    {
+        // Whenever the version changes, we wipe all three caches.
+        // That's useful to, for example, get rid of old unknown
+        // artist images from the remote server. Changing the version
+        // is also needed if we ever change the way keys and values
+        // are stored in the caches.
+        qDebug() << "cache version update from" << v.prev_cache_version() << "to" << v.cache_version;
+        clear(Thumbnailer::CacheSelector::all);
+    }
+}
+
 unique_ptr<ThumbnailRequest> Thumbnailer::get_thumbnail(string const& filename,
                                                         QSize const& requested_size)
 {
-    assert(!filename.empty());
+    if (filename.empty())
+    {
+        throw unity::InvalidArgumentException("Thumbnailer::get_thumbnail(): filename is empty");
+    }
 
     try
     {
@@ -781,9 +850,9 @@ unique_ptr<ThumbnailRequest> Thumbnailer::get_album_art(string const& artist,
                                                         string const& album,
                                                         QSize const& requested_size)
 {
-    if (artist.empty() && album.empty())
+    if (album.empty())
     {
-        throw unity::InvalidArgumentException("Thumbnailer::get_album_art(): both artist and album are empty");
+        throw unity::InvalidArgumentException("Thumbnailer::get_album_art(): album is empty");
     }
 
     try
@@ -802,9 +871,9 @@ unique_ptr<ThumbnailRequest> Thumbnailer::get_artist_art(string const& artist,
                                                          string const& album,
                                                          QSize const& requested_size)
 {
-    if (artist.empty() && album.empty())
+    if (artist.empty())
     {
-        throw unity::InvalidArgumentException("Thumbnailer::get_artist_art(): both artist and album are empty");
+        throw unity::InvalidArgumentException("Thumbnailer::get_artist_art(): artist is empty");
     }
 
     try
@@ -847,12 +916,33 @@ Thumbnailer::CacheVec Thumbnailer::select_caches(CacheSelector selector) const
     return v;
 }
 
+namespace
+{
+
+char const* cache_name(Thumbnailer::CacheSelector selector)
+{
+    switch (selector)
+    {
+        case Thumbnailer::CacheSelector::full_size_cache:
+            return "image cache";
+        case Thumbnailer::CacheSelector::thumbnail_cache:
+            return "thumbnail cache";
+        case Thumbnailer::CacheSelector::failure_cache:
+            return "failure cache";
+        default:
+            return "all caches";
+    }
+}
+
+}
+
 void Thumbnailer::clear_stats(CacheSelector selector)
 {
     for (auto c : select_caches(selector))
     {
         c->clear_stats();
     }
+    qDebug() << "reset statistics for" << cache_name(selector);
 }
 
 void Thumbnailer::clear(CacheSelector selector)
@@ -861,14 +951,25 @@ void Thumbnailer::clear(CacheSelector selector)
     {
         c->invalidate();
     }
+    if (selector == Thumbnailer::CacheSelector::failure_cache ||
+        selector == Thumbnailer::CacheSelector::all)
+    {
+        // Force retry on next remote retrieval even if we
+        // are still within the timeout period.
+        backoff_.set_last_fail_time(chrono::system_clock::time_point())
+                .set_backoff_period(chrono::seconds(0));
+    }
+    qDebug() << "cleared" << cache_name(selector);
 }
 
 void Thumbnailer::compact(CacheSelector selector)
 {
+    qDebug() << "compacting" << cache_name(selector);
     for (auto c : select_caches(selector))
     {
         c->compact();
     }
+    qDebug() << "completed compacting" << cache_name(selector);
 }
 
 }  // namespace internal
