@@ -17,10 +17,12 @@
  *              Michi Henning <michi.henning@canonical.com>
  */
 
+#include <internal/env_vars.h>
 #include <internal/image.h>
 #include <internal/raii.h>
 #include "utils/artserver.h"
 #include "utils/dbusserver.h"
+#include "utils/env_var_guard.h"
 
 #include <boost/algorithm/string/predicate.hpp>
 #include <gtest/gtest.h>
@@ -72,8 +74,8 @@ protected:
         tempdir.reset(new QTemporaryDir(TESTBINDIR "/dbus-test.XXXXXX"));
         setenv("XDG_CACHE_HOME", (tempdir->path() + "/cache").toUtf8().data(), true);
 
-        // set 3 seconds as max idle time
-        setenv("THUMBNAILER_MAX_IDLE", "1000", true);
+        setenv(MAX_IDLE, "1000", true);
+        setenv(LOG_LEVEL, "2", true);  // For coverage
 
         dbus_.reset(new DBusServer());
     }
@@ -88,7 +90,7 @@ protected:
         dbus_.reset();
         art_server_.reset();
 
-        unsetenv("THUMBNAILER_MAX_IDLE");
+        unsetenv(MAX_IDLE);
         unsetenv("XDG_CACHE_HOME");
         tempdir.reset();
     }
@@ -100,10 +102,10 @@ protected:
 
 TEST_F(DBusTest, get_album_art)
 {
-    QDBusReply<QDBusUnixFileDescriptor> reply =
+    QDBusReply<QByteArray> reply =
         dbus_->thumbnailer_->GetAlbumArt("metallica", "load", QSize(24, 24));
     assert_no_error(reply);
-    Image image(reply.value().fileDescriptor());
+    Image image(reply.value());
     EXPECT_EQ(24, image.width());
     EXPECT_EQ(24, image.width());
 }
@@ -113,10 +115,10 @@ TEST_F(DBusTest, get_artist_art)
     // We do this twice, so we get a cache hit on the second try.
     for (int i = 0; i < 2; ++i)
     {
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetArtistArt("metallica", "load", QSize(24, 24));
         assert_no_error(reply);
-        Image image(reply.value().fileDescriptor());
+        Image image(reply.value());
         EXPECT_EQ(24, image.width());
         EXPECT_EQ(24, image.width());
     }
@@ -125,11 +127,11 @@ TEST_F(DBusTest, get_artist_art)
 TEST_F(DBusTest, thumbnail_image)
 {
     const char* filename = TESTDATADIR "/testimage.jpg";
-    QDBusReply<QDBusUnixFileDescriptor> reply =
+    QDBusReply<QByteArray> reply =
         dbus_->thumbnailer_->GetThumbnail(filename, QSize(256, 256));
     assert_no_error(reply);
 
-    Image image(reply.value().fileDescriptor());
+    Image image(reply.value());
     EXPECT_EQ(256, image.width());
     EXPECT_EQ(160, image.height());
 }
@@ -140,11 +142,11 @@ TEST_F(DBusTest, song_image)
     for (int i = 0; i < 2; ++i)
     {
         const char* filename = TESTDATADIR "/testsong.ogg";
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetThumbnail(filename, QSize(256, 256));
         assert_no_error(reply);
 
-        Image image(reply.value().fileDescriptor());
+        Image image(reply.value());
         EXPECT_EQ(200, image.width());
         EXPECT_EQ(200, image.height());
     }
@@ -156,11 +158,11 @@ TEST_F(DBusTest, video_image)
     for (int i = 0; i < 2; ++i)
     {
         const char* filename = TESTDATADIR "/testvideo.ogg";
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetThumbnail(filename, QSize(256, 256));
         assert_no_error(reply);
 
-        Image image(reply.value().fileDescriptor());
+        Image image(reply.value());
         EXPECT_EQ(256, image.width());
         EXPECT_EQ(144, image.height());
     }
@@ -169,7 +171,7 @@ TEST_F(DBusTest, video_image)
 TEST_F(DBusTest, thumbnail_no_such_file)
 {
     const char* no_such_file = TESTDATADIR "/no-such-file.jpg";
-    QDBusReply<QDBusUnixFileDescriptor> reply =
+    QDBusReply<QByteArray> reply =
         dbus_->thumbnailer_->GetThumbnail(no_such_file, QSize(256, 256));
     EXPECT_FALSE(reply.isValid());
     auto message = reply.error().message().toStdString();
@@ -178,12 +180,24 @@ TEST_F(DBusTest, thumbnail_no_such_file)
 
 TEST_F(DBusTest, server_error)
 {
-    QDBusReply<QDBusUnixFileDescriptor> reply =
-        dbus_->thumbnailer_->GetArtistArt("error", "500", QSize(256, 256));
-    EXPECT_FALSE(reply.isValid());
-    auto message = reply.error().message().toStdString();
-    // TODO: That's a seriously poor error message.
-    EXPECT_TRUE(boost::contains(message, "fetch() failed")) << message;
+    {
+        QDBusReply<QDBusUnixFileDescriptor> reply =
+            dbus_->thumbnailer_->GetArtistArt("error", "500", QSize(256, 256));
+        EXPECT_FALSE(reply.isValid());
+        auto message = reply.error().message().toStdString();
+        EXPECT_EQ("Handler::createFinished(): could not get thumbnail for artist: error/500 (256,256): TEMPORARY ERROR",
+                  message) << message;
+    }
+
+    // Again, so we cover the network retry limit case.
+    {
+        QDBusReply<QDBusUnixFileDescriptor> reply =
+            dbus_->thumbnailer_->GetArtistArt("error", "500", QSize(256, 256));
+        EXPECT_FALSE(reply.isValid());
+        auto message = reply.error().message().toStdString();
+        EXPECT_EQ("Handler::checkFinished(): no artwork for artist: error/500 (256,256): TEMPORARY ERROR",
+                  message) << message;
+    }
 }
 
 TEST_F(DBusTest, duplicate_requests)
@@ -218,7 +232,7 @@ TEST_F(DBusTest, rate_limit_requests)
     // rate limited, but it does exercise the code paths as shown by
     // the coverage report.
     int const N_REQUESTS = 10;
-    QDBusPendingReply<QDBusUnixFileDescriptor> replies[N_REQUESTS];
+    QDBusPendingReply<QByteArray> replies[N_REQUESTS];
 
     for (int i = 0; i < N_REQUESTS; i++)
     {
@@ -245,7 +259,7 @@ TEST_F(DBusTest, test_inactivity_exit)
                         static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished));
 
     // start a query
-    QDBusReply<QDBusUnixFileDescriptor> reply =
+    QDBusReply<QByteArray> reply =
         dbus_->thumbnailer_->GetThumbnail(filename, QSize(256, 256));
     assert_no_error(reply);
 
@@ -260,6 +274,11 @@ TEST_F(DBusTest, test_inactivity_exit)
 
 TEST_F(DBusTest, service_exits_if_run_twice)
 {
+    if (!SLOW_TESTS)
+    {
+        return;
+    }
+
     // Try to start a second copy of the thumbnailer service
     QProcess process;
     process.setStandardInputFile(QProcess::nullDevice());
@@ -291,10 +310,15 @@ TEST_F(DBusTest, service_exits_if_name_taken)
 
 TEST(DBusTestBadIdle, env_variable_bad_value)
 {
+    if (!SLOW_TESTS)
+    {
+        return;
+    }
+
     QTemporaryDir tempdir(TESTBINDIR "/dbus-test.XXXXXX");
     setenv("XDG_CACHE_HOME", (tempdir.path() + "/cache").toUtf8().data(), true);
 
-    setenv("THUMBNAILER_MAX_IDLE", "bad_value", true);
+    EnvVarGuard ev_guard(MAX_IDLE, "bad_value");
     try
     {
         unique_ptr<DBusServer> dbus(new DBusServer());
@@ -305,15 +329,19 @@ TEST(DBusTestBadIdle, env_variable_bad_value)
         string err = e.what();
         EXPECT_TRUE(err.find("failed to appear on bus"));
     }
-    unsetenv("THUMBNAILER_MAX_IDLE");
 }
 
 TEST(DBusTestBadIdle, env_variable_out_of_range)
 {
+    if (!SLOW_TESTS)
+    {
+        return;
+    }
+
     QTemporaryDir tempdir(TESTBINDIR "/dbus-test.XXXXXX");
     setenv("XDG_CACHE_HOME", (tempdir.path() + "/cache").toUtf8().data(), true);
 
-    setenv("THUMBNAILER_MAX_IDLE", "999", true);
+    EnvVarGuard ev_guard(MAX_IDLE, "999");
     try
     {
         unique_ptr<DBusServer> dbus(new DBusServer());
@@ -324,7 +352,6 @@ TEST(DBusTestBadIdle, env_variable_out_of_range)
         string err = e.what();
         EXPECT_TRUE(err.find("failed to appear on bus"));
     }
-    unsetenv("THUMBNAILER_MAX_IDLE");
 }
 
 TEST(DBusTestBadIdle, default_timeout)
@@ -332,7 +359,7 @@ TEST(DBusTestBadIdle, default_timeout)
     QTemporaryDir tempdir(TESTBINDIR "/dbus-test.XXXXXX");
     setenv("XDG_CACHE_HOME", (tempdir.path() + "/cache").toUtf8().data(), true);
 
-    unsetenv("THUMBNAILER_MAX_IDLE");
+    EnvVarGuard ev_guard(MAX_IDLE, nullptr);
     unique_ptr<DBusServer> dbus(new DBusServer());  // For coverage with default timeout.
 }
 
@@ -437,10 +464,10 @@ TEST_F(DBusTest, stats)
 
     // Get a remote image from the cache, so the stats change.
     {
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetAlbumArt("metallica", "load", QSize(24, 24));
         assert_no_error(reply);
-        Image image(reply.value().fileDescriptor());
+        Image image(reply.value());
         EXPECT_EQ(24, image.width());
         EXPECT_EQ(24, image.width());
     }
@@ -501,10 +528,10 @@ TEST_F(DBusTest, stats)
 
     // Get the same image again, so we get a hit.
     {
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetAlbumArt("metallica", "load", QSize(24, 24));
         assert_no_error(reply);
-        Image image(reply.value().fileDescriptor());
+        Image image(reply.value());
         EXPECT_EQ(24, image.width());
         EXPECT_EQ(24, image.width());
     }
@@ -534,7 +561,7 @@ TEST_F(DBusTest, stats)
 
     // Get a non-existent remote image from the cache, so the failure stats change.
     {
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetAlbumArt(
                 "no_such_artist", "no_such_album", QSize(24, 24));
     }
@@ -551,7 +578,7 @@ TEST_F(DBusTest, stats)
 
     // Get the same non-existent remote image again, so we get a hit.
     {
-        QDBusReply<QDBusUnixFileDescriptor> reply =
+        QDBusReply<QByteArray> reply =
             dbus_->thumbnailer_->GetAlbumArt(
                 "no_such_artist", "no_such_album", QSize(24, 24));
     }
@@ -589,7 +616,7 @@ int main(int argc, char** argv)
 
     setenv("GSETTINGS_BACKEND", "memory", true);
     setenv("GSETTINGS_SCHEMA_DIR", GSETTINGS_SCHEMA_DIR, true);
-    setenv("TN_UTILDIR", TESTBINDIR "/../src/vs-thumb", true);
+    setenv(UTIL_DIR, TESTBINDIR "/../src/vs-thumb", true);
     ::testing::InitGoogleTest(&argc, argv);
     return RUN_ALL_TESTS();
 }
