@@ -16,6 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
+#include <internal/env_vars.h>
 #include <internal/thumbnailer.h>
 
 #include <testsetup.h>
@@ -29,7 +30,7 @@
 using namespace std;
 using namespace unity::thumbnailer::internal;
 
-#define TEST_SONG TESTDATADIR "/testsong.ogg"
+#define TEST_VIDEO TESTDATADIR "/testvideo.ogg"
 
 // The thumbnailer uses g_get_user_cache_dir() to get the cache dir, and
 // glib remembers that value, so changing XDG_CACHE_HOME later has no effect.
@@ -66,23 +67,14 @@ TEST_F(ThumbnailerTest, slow_vs_thumb)
 {
     Thumbnailer tn;
 
-    auto request = tn.get_thumbnail(TEST_SONG, QSize());
+    auto request = tn.get_thumbnail(TEST_VIDEO, QSize(100, 0));
     EXPECT_EQ("", request->thumbnail());
 
     QSignalSpy spy(request.get(), &ThumbnailRequest::downloadFinished);
     request->download();
     ASSERT_TRUE(spy.wait(15000));  // Slow vs-thumb will get killed after 10 seconds.
 
-    try
-    {
-        request->thumbnail();
-        FAIL();
-    }
-    catch (unity::ResourceException const& e)
-    {
-        string msg = e.what();
-        EXPECT_NE(string::npos, msg.find("did not return after 10000 milliseconds")) << msg;
-    }
+    EXPECT_EQ("", request->thumbnail());
 }
 
 int main(int argc, char** argv)
@@ -90,7 +82,7 @@ int main(int argc, char** argv)
     QCoreApplication app(argc, argv);
 
     // Run fake vs-thumb that does nothing for 20 seconds.
-    setenv("TN_UTILDIR", TESTSRCDIR "/slow-vs-thumb/slow", true);
+    setenv(UTIL_DIR, TESTSRCDIR "/slow-vs-thumb/slow", true);
     ::testing::InitGoogleTest(&argc, argv);
     return RUN_ALL_TESTS();
 }
